/*
 *  ofxQTKitMoviePlayer example
 *
 * Created by James George, http://www.jamesgeorge.org
 * over a long period of time for a few different projects in collaboration with
 * Flightphase http://www.flightphase.com 
 * and the rockwell group lab http://lab.rockwellgroup.com
 *
 **********************************************************
 *
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without
 * restriction, including without limitation the rights to use,
 * copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following
 * conditions:
 *
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
 * HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 *
 * ----------------------
 * 
 * ofxQTKitVideoPlayer provides QTKit/CoreVideo accelerated movie playback
 * for openFrameworks on OS X
 * 
 * This class replaces almost all of the functionality of ofVideoPlayer on OS X
 * and uses the modern QTKit and CoreVideo libraries for playback
 *
 * Thanks to Marek Bereza for his initial QTKitVideoTexture
 * http://www.mrkbrz.com/
 *
 * Thanks to Anton Marini for help wrangling buffer contexts
 * http://vade.info/ 
 */

#include "testApp.h"
int videoWidth=0;
int videoHeight=0;
//contourFinder params
int minArea = 200;
int maxArea = 100000;
int numConsidered = 80;
bool findHoles = false;


int blur = 11;
int threshold =19;
int threadLimit = 1;
bool doDrawVideo = true;
bool doParticles = false;
float largestBlob = 0.0;
bool isBackgroundSet = false;
//--------------------------------------------------------------
void testApp::setup(){
	
	ofSetVerticalSync(true);
	ofEnableAlphaBlending();
	
	//try out different modes!
	//mode = OFXQTVIDEOPLAYER_MODE_TEXTURE_ONLY;
	mode = OFXQTVIDEOPLAYER_MODE_PIXELS_ONLY;
	//mode = OFXQTVIDEOPLAYER_MODE_PIXELS_AND_TEXTURE;
	
	videoPlayer.loadMovie("synchronizedswim.mp4", mode);	
	videoPlayer.play();
	videoWidth = videoPlayer.getWidth();
    videoHeight =  videoPlayer.getHeight();
    
    cvColorImage.allocate(videoWidth, videoHeight);
	cvGrayImageFromColorImage.allocate(videoWidth, videoHeight);
	cvLastGrayImageFromColorImage.allocate(videoWidth, videoHeight);
	cvGrayBackground.allocate(videoWidth, videoHeight);
	if(mode != OFXQTVIDEOPLAYER_MODE_TEXTURE_ONLY){
		ourOwnTexture.allocate(videoPlayer.getWidth(), videoPlayer.getHeight(), GL_RGBA);
	}
	for(int i=0; i<threadLimit; i++)
	{
		ContourFinderThread *thread =new ContourFinderThread();
		thread->nID = i;
		thread->blur = blur;
		thread->minArea = minArea;
		thread->maxArea = maxArea;
		thread->numConsidered = numConsidered;
		thread->findHoles = findHoles;
		threads.push_back(thread);
        thread->initAndSleep();
	}
	
}

//--------------------------------------------------------------
void testApp::update(){
	trackedBlobs.clear();
	videoPlayer.update();
	if(videoPlayer.isFrameNew()){
		//if we have created our video player with pixels, we can use make an oftexture
		//with any custom pixel processing we want
		//these modes are how to integrate ofxQTKitVideoPlayer with openCV
		if(mode == OFXQTVIDEOPLAYER_MODE_PIXELS_ONLY){
			unsigned char* moviePixels = videoPlayer.getPixels();
			 cvColorImage.setFromPixels(moviePixels, videoWidth, videoHeight);
			if (!isBackgroundSet) 
			{
				cvGrayBackground = cvColorImage;
				isBackgroundSet = true;
			}
			//process pixels here if you want
			//...
			//load them into our own ofTexture to draw
			ourOwnTexture.loadData(moviePixels, videoPlayer.getWidth(), videoPlayer.getHeight(), GL_RGBA);
			for(int i=0; i<threads.size(); i++)
			{
				if(threads[i]->isReady)
				{
					vector<ofxCvBlob>  blobs = threads[i]->contourFinder->blobs;
					cout << "blobs.size(): " << blobs.size() << endl;
					for (int j=0; j<blobs.size(); j++) 
					{
						trackedBlobs.push_back(blobs[j]);
						if (blobs[j].area>largestBlob)
						{
							largestBlob = blobs[j].area;
						}
					}
					threads[i]->contourFinder->blobs.clear();
				}
				
			}
			blobTracker.trackBlobs( trackedBlobs );
			for(int i=0; i<threads.size(); i++)
			{
				if(threads[i]->isReady)
				{
					threads[i]->cvColorImageAlpha = cvColorImage;
					threads[i]->cvBackgroundImageDiff = cvGrayBackground;
					threads[i]->updateThread();
				}
				
			} 
			
		}
	}
}

//--------------------------------------------------------------
void testApp::draw(){
	ofBackground(0,0,0);
	if(mode == OFXQTVIDEOPLAYER_MODE_PIXELS_ONLY){
		ourOwnTexture.draw(0,0);	
	}
	else {
		videoPlayer.draw(0,0);
	}
	
	ofSetColor(255, 255, 255);	
	char format[1024];
	sprintf(format, "playback framerate: %f\nmovie is size %f x %f\nat time: %f/%f\non frame %d / %d\npercent done: %f \nLoops? %s\nDone Yet? %s",  
			ofGetFrameRate(), 
			videoPlayer.getWidth(), videoPlayer.getHeight(),
			videoPlayer.getPositionInSeconds(), videoPlayer.getDuration(),
			videoPlayer.getCurrentFrame(), videoPlayer.getTotalNumFrames(),
			videoPlayer.getPosition()*100,
			(videoPlayer.getMovieLoopState() ? "yes" : "no"), (videoPlayer.getIsMovieDone() ? "yes" : "no") );
	
	ofDrawBitmapString(format, 20, videoPlayer.getHeight()+15);
	for( int i=0; i<trackedBlobs.size(); i++ ) 
	{
        ostringstream docstring;
        docstring <<"area: " << trackedBlobs[i].area << endl;
        //docstring << findOrder(blobs[i].id) << endl;
        ofDrawBitmapString( docstring.str(),
						   trackedBlobs[i].centroid.x, trackedBlobs[i].centroid.y );
		ofPushStyle();
			trackedBlobs[i].draw();
		ofPopStyle();
    }
}

//--------------------------------------------------------------
void testApp::keyPressed(int key){
	
	//random testing functions
	if(key == 'p'){
		if(videoPlayer.isPaused()){
			videoPlayer.play();
		}
		else {
			videoPlayer.pause();
		}
	}

	if(key == 'm'){
		//move play head to the middle
		videoPlayer.setPosition(.5);
	}
	
	if(key == 'f'){
		//test movie set frame frame
		videoPlayer.setFrame(100);
	}
	
	if(key == 'l'){
		//toggle loop
		videoPlayer.setLoopState(!videoPlayer.getMovieLoopState());
	}	
}

//--------------------------------------------------------------
void testApp::keyReleased(int key){

}

//--------------------------------------------------------------
void testApp::mouseMoved(int x, int y ){

}

//--------------------------------------------------------------
void testApp::mouseDragged(int x, int y, int button){

}

//--------------------------------------------------------------
void testApp::mousePressed(int x, int y, int button){

}

//--------------------------------------------------------------
void testApp::mouseReleased(int x, int y, int button){

}

//--------------------------------------------------------------
void testApp::windowResized(int w, int h){

}

